% Example 2 E coli core
% Complete workflow to sample loopless solutions of a metabolic model
% Inputs:  (-)
% Outputs: (-)
% Updates:
%         15/03/16  Faster model pre-processing and more efficient ll-FVA
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc, addpath('fxns_OCTAVE/');

%% Step 0. Load model and define sampling and optmization parameters
filename       = 'ecoli_core_OCTAVE';   % Examples: 'toyModel_OCTAVE', 'ecoli_core_OCTAVE', 'iRBC283_OCTAVE'
load([filename,'.mat']);                % Model structure requires the following fields: S matrix (full), lb, ub, description (model name)
numSamples     = 1e2;                   % Number of samples to be generated
numDiscarted   = 1e1;                   % Samples discarded
stepsPerPoint  = 1e2;                   % Steps per point (thinning)
method         = 'll_ACHRB';            % 'll_ACHRB' or 'ACHR'
warmUpMethod   = 'loopless';            % 'loopless' or 'conventional'
params.tolint  = 1e-6;                  % Optimization parameters (GLPK)
params.tolobj  = 1e-6;
params.tolbnd  = 1e-6;    
params.toldj   = 1e-6; 
params.msglev  = 0;                     % No output messages

%% Step 1. Build loopless structure and pre-process model using ll-FVA
% (Consider checking numerical tractability of the model before progressing to Step 2. For example, the fxn
% compactCbModel() compacts de model structure by lumping linear reactions. Warning: if the model is very
% ill-conditioned this fxn might not generate a better conditioned model)
disp('Reducing model...');
modelLP   = linearStructure_OCTAVE(model);
modelLP   = looplessFVA_OCTAVE(modelLP,'linear',params);
initModel = looplessStructureMILP_OCTAVE(modelLP);
disp(['Loopless structure succesfully loaded: rxns = ',num2str(size(initModel.S,2)),', mets = ',num2str(size(initModel.S,1))]);
modelSampling = looplessFVA_OCTAVE(initModel,'loopless',params);
disp('ll-FVA succesfully run');
disp(['Model to be sampled: rxns = ',num2str(size(modelSampling.S,2)),', mets = ',num2str(size(modelSampling.S,1))]);

% Step 2. Generate warmup-points
disp('Generating warmup points...');
modelSampling = warmUpLoopless_OCTAVE(modelSampling,2*modelSampling.numRxns,warmUpMethod,params); 

% Set-up parameters for each sampler: ACHR and ll-ACHR/ACHRB
centerPoint     = modelSampling.centroid;

% Check whether the there are potentially active loop laws
if ~isempty(modelSampling.Nint)
    nonZeroElements = sum((modelSampling.Nint~=0),2);
    loopMatrix      = sign(modelSampling.Nint);
    
    % Check feasibility of the initial point
    checkPattern = diag(sign(centerPoint(modelSampling.internal)));
    
    % If centroid unfeasible, find closest feasible point
    if any(sum(loopMatrix*checkPattern,2)==nonZeroElements) || any(sum(loopMatrix*checkPattern,2)==-nonZeroElements)
        modelSampling.centroid = getInteriorPoint_OCTAVE(modelSampling,.95,params);
        disp('Feasible initial point found...');
    end
end
%% Step 3. Loopless sampling
looplessACHRBsampler_OCTAVE(modelSampling,numSamples,filename,method,numDiscarted,stepsPerPoint);
clear all