function modelFinal = compactCbModel_OCTAVE(modelSampling)
% Compacts constrained-based model structure by lumping pathways into
% single reactions. Warning: if the model is very ill-conditioned this fxn
% might not generate a better conditioned model (OCTAVE)
% Inputs:  model sampling structure
%
% Outputs: compacted model structure
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform compaction
modelRed = linearStructure_OCTAVE(modelSampling);
[Sset,SetLb,SetUb,ReactionSet,MetaboliteSet] = findReactionSets(modelRed.S,modelRed.lb,modelRed.ub);

% Build final model structure
modelCompacted.S        = Sset;
modelCompacted.lb       = SetLb;
modelCompacted.ub       = SetUb;
modelCompacted.c        = zeros(size(SetLb));
modelCompacted.b        = zeros(size(Sset,1),1);
modelCompacted.description = [modelRed.description,'_compacted'];
modelCompacted.numRxns = length(modelCompacted.c); 

% Run consistency check using FVA
modelFinal = looplessStructureMILP_OCTAVE(modelCompacted);
try
    modelFinal = looplessFVA_OCTAVE(modelFinal);
catch
   disp('Compaction failed. Consider revising the model');
end