function modGurobi = looplessStructureMILP_GLPK(model)
% Builds loopless structure problem (OCTAVE)
% Inputs:  model structure, flag (optional)
% Outputs: loopless model structure
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[l,n] = size(model.S);

% Useful information for later
model = parseInternalRxns(model);

% Reorganize the stoichiometric matrix as [internalRxns|exchangeRxns]
modGurobi.numRxns  = n;
modGurobi.internal = model.internal;
modGurobi.exchange = model.exchange;
modGurobi.S        = model.S;
modGurobi.rxns     = model.rxns;
modGurobi.rxnNames = model.rxnNames;
modGurobi.mets     = model.mets;
modGurobi.metNames = model.metNames;
modGurobi.rev      = (model.lb<0).*(model.ub>0);
modGurobi.description = model.description;

% Calculate null basis of Sint
p = 0; m = length(modGurobi.internal);
looplessConstraints = [];
modGurobi.Nint = sparsifyNint(model,'glpk')';
if ~isempty(modGurobi.Nint)           % If Nint is null, then fill with zeros    
    [p,m] = size(modGurobi.Nint);
    looplessConstraints = [zeros(p,n),modGurobi.Nint,zeros(p,m)];
end

% Determine constrained internal reactions
M = eye(n);
M(modGurobi.exchange,:) = [];

% Define binary constant, model objective and sense
K = 1e3;
modGurobi.obj   = zeros(n+2*m,1);
modGurobi.sense = -1; % maximization problem

% Constraints definition
modGurobi.A = [model.S,zeros(l,2*m);...
    looplessConstraints;...
    zeros(m,n),eye(m),(1+K)*eye(m);...
    zeros(m,n),-eye(m),-(1+K)*eye(m);...
    M,zeros(m),-diag(model.ub(modGurobi.internal));...
    -M,zeros(m),-diag(model.lb(modGurobi.internal))];

% RHS model
modGurobi.rhs = [zeros(l+p,1);K*ones(m,1);-ones(m,1);zeros(m,1);-model.lb(modGurobi.internal)];

% Sign assignation
modGurobi.ctype = blanks(l+p+4*m);
for i = 1:l+p+4*m
    if i <= l+p
        modGurobi.ctype(i) = 'S';
    else
        modGurobi.ctype(i) = 'U';
    end
end

% Assignation of variable types
modGurobi.vartype = blanks(n+2*m);
for i = 1:n+2*m
    if i <= n+m
        modGurobi.vartype(i) = 'C';
    else
        modGurobi.vartype(i) = 'I';
    end
end

% Bounds assignation 
modGurobi.lb  = [model.lb;-K*ones(m,1);zeros(m,1)];
modGurobi.ub  = [model.ub;K*ones(m,1);ones(m,1)];