function modelSNP = nullSparseBasisStructure_GLPK(model,x)
% Formulate Null-space Basis Problem
% Inputs:     model structure, multiplicative factor w'*P = x
%
% Outputs:    Nsnp (sparse null space matrix for internal loops)
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2016 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameter initialization
[m,n] = size(model.S);

% Definition of model structure
modelSNP.A = sparse([model.S,zeros(m,n);...
    -eye(n),eye(n);...
    eye(n),eye(n);...
    x,zeros(1,n)]);

% Redefine LP problem:
% (1) RHS
modelSNP.rhs = zeros(m+2*n+1,1);

% (2) Sense
modelSNP.ctype = blanks(m+2*n+1);
for ix = 1:m+2*n+1
    if ix<=m
        modelSNP.ctype(ix) = 'S';
    else
        modelSNP.ctype(ix) = 'L';
    end
end

% (3) Bounds
modelSNP.lb = [model.lb;-1e3*ones(n,1)];
modelSNP.ub = [model.ub;1e3*ones(n,1)];

% (4) Model sense
modelSNP.sense = 1;

% (5) Variables
modelSNP.vartype = blanks(2*n);
for ix = 1:2*n
    modelSNP.vartype(ix) = 'C';
end

% (6) Obj fxn
modelSNP.obj = [zeros(n,1);ones(n,1)];