function parallelSampler(modelSampling,numSamples,numWorkers,numDiscarted,stepsPerPoint,filename,method,shuffleFlag)
% Complete workflow to sample loopless solutions of a metabolic model
% Inputs:  model sampling structure, numSamples, numWorkers, numDiscarted,
%          stepsPerPoint, filename, method, shuffleFlag (optional)
% Outputs: (-)
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Hit-And-Run sampling
addpath('fxns/');
if nargin<8; shuffleFlag = 1; end;
if shuffleFlag            % shuffle random number generator stream
    try 
        rng('shuffle');   % new MATLAB versions
    catch
        stream = RandStream('mt19937ar','seed',sum(100*clock)); % for older versions
        RandStream.setDefaultStream(stream);
    end
end
matlabRel = version('-release');
matlabRel = str2double(matlabRel(regexp(matlabRel,'\d')));
try    
    if (matlabRel>2013); parpool(numWorkers);        
    else matlabpool(numWorkers); end;  % For older Matlab versions
catch
    if (matlabRel>2013); delete(gcp('nocreate')); parpool(numWorkers);
    else matlabpool('close'); matlabpool(numWorkers); end;  % For older Matlab versions
end
samplesPerWorker          = numSamples/numWorkers;
points{numWorkers}        = [];
rejectionRate{numWorkers} = [];
samplingTime{numWorkers}  = [];
parfor ix = 1:numWorkers   
    [points{ix},samplingTime{ix},rejectionRate{ix}] = looplessACHRBsamplerParallel(modelSampling,samplesPerWorker,ix,method,numDiscarted,stepsPerPoint);
end
if (matlabRel>2013); delete(gcp('nocreate'));
else matlabpool('close'); end;  % For older Matlab versions
samplePoints = [];
modelSampling.samplingTime  = [];
modelSampling.rejectionRate = [];
for ix = 1:numWorkers
    samplePoints = [samplePoints,points{ix}];
    modelSampling.rejectionRate = [modelSampling.rejectionRate;rejectionRate{ix}];
    modelSampling.samplingTime  = [modelSampling.samplingTime,samplingTime{ix}];
end
clearvars -except modelSampling samplePoints filename method
save([filename,'_warmup_',modelSampling.warmupMethod,'_sampler_',method,'_parallel'],'-v7.3');