function modelFinal = compactCbModel(modelSampling)
% Compacts constrained-based model structure by lumping pathways into
% single reactions. Warning: if the model is very ill-conditioned this fxn
% might not generate a better conditioned model
% Inputs:  model sampling structure
%
% Outputs: compacted model structure
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2015 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform compaction
modelRed = linearStructure(modelSampling);
[Sset,SetLb,SetUb,ReactionSet,MetaboliteSet] = findReactionSets(full(modelRed.S),modelRed.lb,modelRed.ub);
disp('out')
% Match reaction names and only include rxns not associated to biomass
numRxns = size(SetLb,1);
rxns{numRxns,1} = [];
counter = 1;
while counter<=numRxns
    rxnIdx = find(ReactionSet(:,1)==counter);
    
    % If the reaction has not been lumped
    if length(rxnIdx)==1
        rxns{counter} = modelRed.rxns{rxnIdx};
    else
        % Find reactions with the same identifier
        for j = 1:length(rxnIdx)
            if j == 1
                tempName = modelRed.rxns{rxnIdx(j)};
            else
                tempName = [tempName,'_',modelRed.rxns{rxnIdx(j)}];
            end
        end
        rxns{counter} = tempName;
    end
    counter = counter+1;
end

% Build final model structure
modelCompacted.S        = sparse(Sset);
modelCompacted.lb       = SetLb;
modelCompacted.ub       = SetUb;
modelCompacted.c        = zeros(size(SetLb));
modelCompacted.b        = zeros(size(Sset,1),1);
modelCompacted.rxns     = rxns;
modelCompacted.rxnNames = rxns;
modelCompacted.mets     = modelRed.mets(MetaboliteSet~=-1);
modelCompacted.metNames = modelCompacted.mets;
modelCompacted.description = [modelRed.description,'_compacted'];
modelCompacted.numRxns = length(modelCompacted.c); 

% Run consistency check using FVA
% modelFinal = looplessStructureMILP(modelCompacted);
modelFinal = modelCompacted;
return
try
    modelFinal = looplessFVA(modelFinal);
catch
   disp('Compaction failed. Consider revising the model');
end